use strict;
use warnings;
use File::Basename;

my $program_version = "1.0";
my ($program, $dirpath) = fileparse($0);

my $usage =<<USAGE;

Run one or more jobs in a sequence while monitoring their runtime and max RAM usage with
/usr/bin/time. Job(s) can be listed directly in commandline or loaded from a text file

 Running jobs listed in commandline :
 ====================================
    perl $program -l <log file path> -cmd <commandline of job #1> -cmd <commandline of job #2 > ..
    
    Runs the job(s) preceded by -cmd and saves their runtime and maximum RAM usage stats in the
    log file. These stats are calculated by /usr/bin/time.
       
    Example : perl $program log.txt -c perl foo.pl bar.txt > a.txt -c grep 'huh?' a.txt 

 Running jobs listed in text file :
 ==================================     
    perl $program -l <log file path> -i <text file containing commandlines>
    
    The text file must contain one commandline (job) per line. For example:
    perl foo.pl bar.txt > a.txt
    grep 'huh?' a.txt
    
    I skip empty lines and lines starting with "#" so you can have comments and spacing

USAGE

die $usage unless @ARGV;

my $usr_bin_time_exists = (-e "/usr/bin/time");
die "Did not detect /usr/bin/time on this machine !\n" unless $usr_bin_time_exists;


my $commandline = "@ARGV"; my $log_filepath;
if ($commandline =~ /-l\s+(\S+)/){
    $log_filepath = $1;
    $commandline =~ s/-l\s+\S+//;
}
else{
    die "You need to specify a log filepath with -l !\n";
}

#die "Hey, this log_filepath points to an existing file: $log_filepath !\n" if (-e $log_filepath);
open LOG, "> $log_filepath" or die "Failed to create log file at $log_filepath\n";


my @cmds;

if ($commandline =~ /-cmd/){
    @cmds = split /\s*-cmd/, $commandline;
}
elsif ($commandline =~ /-i/){
    $commandline =~ s/-i//;
    open IN , "< $commandline" or die "Failed to open $commandline !\n";
    while (defined (my $line = <IN>)){
        chomp $line;
        push @cmds, $line if ($line && ($line !~ /^#/)); # skip empty lines and lines starting with '#'
    }
    close IN;
}
else{
    die "You need to list job cmds in commandline with -cmd or in a file pointed to by -i !\n";
}

die "You did not specify any job commandlines !" unless @cmds;

# Go thru @cmds and remove any empty slots
my @real_cmds;
foreach (@cmds){
    push @real_cmds, $_ if ($_ =~ /\S+/);
}
@cmds = @real_cmds;

foreach my $i (0..$#cmds){
    my $cmd = $cmds[$i];
    $cmd =~ s/-cmd//;
    
    print "\n----------------------------------\n".($i+1).". $cmd\n\n";
    my ($runtime, $memUsage);
        my $current_time_in_secs = time();
        my $temp_raw_binTime_file = "temp_process_stat_$current_time_in_secs.txt";
        system ("(/usr/bin/time -f \"%E %M\" $cmd) 2> $temp_raw_binTime_file");
        die "Failed to create temporary stat file in the current running directory !\n" if (! -e $temp_raw_binTime_file);

        my $stat = `cat $temp_raw_binTime_file`;
        chomp($stat);
        ($runtime, $memUsage) = split /\s/, $stat;
        system("rm $temp_raw_binTime_file");
        #die "\n" unless ($memUsage && ($memUsage =~ /\d+/));
        
        if ($memUsage && ($memUsage =~ /\d+/)){
            # Convert usage to MB
                $memUsage = sprintf ("%.2f", ($memUsage/1024));
            
            # Convert runtime to mins
                if ($runtime =~ /\.(\d+)/){
                    $runtime =~ s/\.\d+//;
                }
            
            my $time_in_mins = 0;
            my @time_parts = split /:/, $runtime;
            if (@time_parts){
                if (@time_parts == 3){
                    $time_in_mins += (60 * $time_parts[0]);
                    $time_in_mins += $time_parts[1];
                    $time_in_mins += ($time_parts[2]/60);
                }
                elsif (@time_parts == 2){
                    $time_in_mins += $time_parts[0];
                    $time_in_mins += ($time_parts[1]/60);
                }
                $time_in_mins = sprintf("%.2f", $time_in_mins); 
            }# end if @time_parts
            else {
                $time_in_mins = -999;
            }
            
            print LOG ($i+1).". $cmd\n";
            print LOG "Runtime = $time_in_mins mins, MAX_RAM_Usage = $memUsage MB\n";
            print "\n\nRuntime = $time_in_mins mins, MAX_RAM_Usage = $memUsage MB\n";
        }
        else{
            print "\nSomething wrong happened with job #".($i+1).". Here's the system report :\n$stat\n";
            print "Skipping to the next cmd ..\n";
        }
    print LOG "--------------------------------------------------------------------\n\n";
    print "\n\n";
}# end foreach cmd 
close LOG;